/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;

import java.util.Properties;

    /** <code>MultiEnum</code> facilitates the enumeration of multiple
        enumerable objects.
<P>
        The class itself implements <code>Enumeration</code> and the
        constructors accept two or more <code>Enumeration</code> objects to use
        as a basis. It will enumerate over all of the elements in each of the
        enumerations in the order specified.
    */
public class MultiEnum implements Enumeration {

    private Vector  enumerations;   // Vector of Enumeration
    private int     enumIndex;      // Which Enumeration are we processing now
    private int     nEnums;         // Total # of enumeration objects

    /** A constructor that deals with the common case of enumerating
        two enumerations.
        @param  enum1   The first enumeration to use
        @param  enum2   The second enumeration to use
    */
    public MultiEnum(Enumeration enum1, Enumeration enum2)
    {
        enumerations = new Vector(2);
        enumerations.addElement(enum1);
        enumerations.addElement(enum2);
        enumIndex = 0;
        nEnums = 2;
    }

    /** A constructor that deals with the common case of enumerating
        three enumerations.
        @param  enum1   The first enumeration to use
        @param  enum2   The second enumeration to use
        @param  enum3   The third enumeration to use
    */
    public MultiEnum(Enumeration enum1, Enumeration enum2, Enumeration enum3)
    {
        enumerations = new Vector(3);
        enumerations.addElement(enum1);
        enumerations.addElement(enum2);
        enumerations.addElement(enum3);
        enumIndex = 0;
        nEnums = 3;
    }

    /** A general purpose constructor that accepts a vector of enumerations.
        @param  enums   A Vector of Enumerations
    */
    public MultiEnum(Vector enums)
    {
        enumIndex = 0;
        enumerations = (Vector)enums.clone();
        nEnums = enumerations.size();
    }

    public boolean hasMoreElements()
    {
        if (enumIndex == nEnums)    // Already finished
            return false;

        Enumeration curEnum =
            (Enumeration)enumerations.elementAt(enumIndex);
        if (curEnum.hasMoreElements())
            return true;
        else {
            enumIndex++;
            return hasMoreElements();
        }
    }

    public Object nextElement()
        throws NoSuchElementException
    {
        if (enumIndex == nEnums)    // Already finished
            throw new NoSuchElementException();

        Enumeration curEnum =
            (Enumeration)enumerations.elementAt(enumIndex);
        try {
            return curEnum.nextElement();
        } catch (NoSuchElementException e) {
            enumIndex++;
            return nextElement();
        }
    }

}








